import * as Database from "./Modules/database.js"
import {
  Player,
  world,
  ItemStack,
  MinecraftItemTypes
} from "@minecraft/server"
const prefix = "!"
const currency = "$"

world.events.beforeChat.subscribe(async data => {
  if (data.message.startsWith(prefix)) {
    let args_ = data.message.slice(prefix.length).trim().split(' ');
    let command = args_.shift().toLowerCase();
    data.cancel = true;
    const player = data.sender;
    const text = data.message;
    customCommand(command, args_, text, player);
  }
})

world.events.playerJoin.subscribe(async data => {
  await getMoney(data.player)
})

world.events.entityHurt.subscribe(async data => {
  let entityHurt = data.hurtEntity
  let lowEntity = [
    "minecraft:pig",
    "minecraft:cow",
    "minecraft:sheep",
    "minecraft:chicken",
    "minecraft:cod",
    "minecraft:salmon",
    "minecraft:pufferfish",
    "minecraft:tropicalfish"
  ]
  let mediumEntity = [
    "minecraft:zombie",
    "minecraft:husk",
    "minecraft:skeleton",
    "minecraft:stray",
    "minecraft:blaze",
    "minecraft:zombie_villager",
    "minecraft:zombie_villager_v2",
    "minecraft:pillager",
    "minecraft:vex",
    "minecraft:evocation_illager",
    "minecraft:slime",
    "minecraft:drowned",
    "minecraft:guardian",
    "minecraft:iron_golem",
    "minecraft:magma_cube",
    "minecraft:cave_spider",
    "minecraft:spider",
    "minecraft:endermite",
    "minecraft:zoglin",
    "minecraft:zombie_pigman"
  ]
  let highEntity = [
    "minecraft:warden",
    "minecraft:phantom",
    "minecraft:creeper",
    "minecraft:enderman",
    "minecraft:wither_skeleton",
    "minecraft:ghast",
    "minecraft:shulker",
    "minecraft:ravager",
    "minecraft:ender_dragon"
  ]
  if (data.cause != "none" && data.damagingEntity.typeId === "minecraft:player" && entityHurt.getComponent("health").current <= 0) {
    let player = data.damagingEntity
    let get = 0
    if (lowEntity.includes(entityHurt.typeId)) {
      get = Math.floor(Math.random() * 5) + 1
    }
    if (mediumEntity.includes(entityHurt.typeId)) {
      get = Math.floor(Math.random() * 15) + 15
    }
    if (highEntity.includes(entityHurt.typeId)) {
      get = Math.floor(Math.random() * 50) + 50
    }
    if (entityHurt.typeId === "minecraft:player") {
      let targetMoney = await getMoney(entityHurt)
      if (targetMoney <= 0) {
        get = 0
      } else {
        get = Math.floor(targetMoney / 3)
      }
      await updateMoney(entityHurt, targetMoney - get)
      entityHurt.tell(`§aYou just killed and lost §e${currency}${get}`)
    }
    if (get <= 0) return;
    let playerMoney = await getMoney(player)
    await updateMoney(player, playerMoney + get)
    player.onScreenDisplay.setActionBar(`§aYou got §e${currency}${get}`)
  }
})

/**
 * Set Custom Command
 * @param {String} command 
 * @param {String} args 
 * @param {String} msg 
 * @param {Player} player 
 */
function customCommand(command, args, msg, player) {
  switch (command.toLowerCase()) {
    case 'help':
      help(player)
      break;
    case 'money':
    case 'balance':
      balance(player, args)
      break;
    case 'pay':
    case 'transfer':
      pay(player, args)
      break;
    case 'topmoney':
    case 'topbalance':
      topMoney(player)
      break;
    case 'setmoney':
    case 'setbalance':
      setMoney(player, args)
      break;
    default:
      player.tell(`§4There's no that command! Use ${prefix}help to get command list.`)
      break;
  }
}

/**
 * @param {String} playerName
 * @returns {Player | undefined}
 */
function getPlayer(playerName) {
  let allPlayers = world.getPlayers()
  let name = playerName.toLowerCase()
  let targetPlayer = undefined;
  for (const p of allPlayers) {
    if (name == p.name.toLowerCase() || name == p.nameTag.toLowerCase() || (name.length >= 3 && p.nameTag.toLowerCase().startsWith(name))) {
      targetPlayer = p
    }
  }
  return targetPlayer
}

// -- [HELP] --

/**
 * @param {Player} player 
 */
function help(player) {
  let helpMsg =
    `§eHelp Message ("?" That means it's optional or not must be filled)
§a Money Command :
§e - §a${prefix}money §e| ${prefix}money <player_name?> (To see your's money or someone's money)
§e - §a${prefix}pay §e| ${prefix}pay <player_name> <amount> (To pay / transfer money to other player)
§e - §a${prefix}topmoney §e| ${prefix}topmoney (To see top money leaderboard)
`
  if (player.hasTag("admin")) {
    helpMsg += `
§a Admin Command :
§e - §a${prefix}setmoney §e| ${prefix}setmoney <player_name> <amount> (To set player's money)
`
  }
  player.tell(helpMsg)
}

// -- [MONEY FUNCTION] --

/**
 * @param {Player} player 
 * @returns {Number}
 */
async function getMoney(player) {
  let playerMoney = await Database.get(`Balance_${player.nameTag}`)
  if (playerMoney == undefined) {
    Database.set(`Balance_${player.nameTag}`, 0)
    return 0
  } else {
    return Number(playerMoney)
  }
}

/**
 * @param {Player} player 
 * @param {String | Number} value 
 */
async function updateMoney(player, value) {
  await Database.set(`Balance_${player.nameTag}`, value)
}

/**
 * @param {Player} player 
 * @param {String} args 
 */
async function balance(player, args) {
  if (!args[0]) {
    let playerMoney = await getMoney(player)
    player.tell(`§aYour's money : §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
  } else {
    let targetPlayer = await getPlayer(args[0])
    if (targetPlayer != undefined) {
      let targetMoney = await getMoney(targetPlayer)
      player.tell(`§a${targetPlayer.nameTag}'s money : §e${currency}${await targetMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    } else {
      let playerMoney = await getMoney(player)
      player.tell(`§aYour's money : §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
    }
  }
}

/**
 * @param {Player} player 
 * @param {String} args 
 */
async function pay(player, args) {
  if (!args[0]) return player.tell("§4You must input the player name!")
  if (!args[1]) return player.tell("§4You must input the amount!")
  let targetPlayer = await getPlayer(args[0])
  if (targetPlayer != undefined) {
    if (targetPlayer == player) return player.tell("§4You can't pay to yourself!")
    let amount = Number(args[1])
    if (amount <= 0) return player.tell("§4You must input the amount more than 1!")
    let playerMoney = await getMoney(player)
    let targetMoney = await getMoney(targetPlayer)
    if (amount > playerMoney) return player.tell("§4You don't have enough money!")
    playerMoney = playerMoney - amount
    targetMoney = targetMoney + amount
    await updateMoney(player, playerMoney)
    await updateMoney(targetPlayer, targetMoney)
    player.tell(`§aSuccessfully paid to ${targetPlayer.nameTag} for §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}.`)
    targetPlayer.tell(`§a${player.nameTag} just paid to you for §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}.`)
  } else {
    player.tell("§4There's no player with that name!")
  }
}

/**
 * @param {Player} player 
 */
async function topMoney(player) {
  let BalanceData = []
  Database.gets().filter(data => {
    if (data.key.startsWith("Balance_")) {
      let playerName = data.key.substring("Balance_".length)
      if (playerName != "undefined") {
        let playerMoney = data.value
        let moneyData = {
          playerName: playerName,
          playerMoney: playerMoney
        }
        BalanceData.push(moneyData)
      }
    }
  })
  if (BalanceData.length <= 0) return player.tell("§4There's no player with money!")
  BalanceData.sort((a, b) => b.playerMoney - a.playerMoney)
  let message = "§a[TOP MONEY LEADERBOARD] :"
  for (let i = 0; i < BalanceData.length; i++) {
    let playerName = BalanceData[i].playerName
    let playerMoney = Number(BalanceData[i].playerMoney)
    message += `\n§a${i + 1}. ${playerName}: §e${currency}${await playerMoney.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`
  }
  player.tell(message)
}

/* -- [ADMIN FUNCTION] */

/**
 * @param {Player} player 
 * @param {String} args 
 */
async function setMoney(player, args) {
  if (!player.hasTag("admin")) return player.tell("§4You're not admin!")
  if (!args[0]) return player.tell("§4You must input the player name!")
  if (!args[1]) return player.tell("§4You must input the amount!")
  let amount = Number(args[1])
  if (!Number.isInteger(amount)) return player.tell("§4You must input number!")
  let targetPlayer = await getPlayer(args[0])
  if (targetPlayer != undefined) {
    await updateMoney(targetPlayer, amount)
    player.tell(`§aSuccessfully set ${targetPlayer.nameTag}'s money to §e${currency}${await amount.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,')}`)
  } else {
    player.tell("§4There's no player with that name!")
  }
}